package nl.kallestruik.dtweaks

import co.aikar.commands.ConditionFailedException
import co.aikar.commands.PaperCommandManager
import nl.kallestruik.dlib.DUtil
import nl.kallestruik.dlib.MathHelper
import nl.kallestruik.dlib.ReflectionUtil
import nl.kallestruik.dtweaks.commands.CommandPlayer
import nl.kallestruik.dtweaks.commands.CommandSpawnzones
import nl.kallestruik.dtweaks.commands.CommandToggletrample
import nl.kallestruik.dtweaks.managers.FakePlayerManager
import nl.kallestruik.dtweaks.managers.NoSpawnZoneManager
import nl.kallestruik.dtweaks.managers.TrampleManager
import nl.kallestruik.dtweaks.managers.TweakManager
import nl.kallestruik.dtweaks.tweaks.craftingtweaks.*
import nl.kallestruik.dtweaks.tweaks.croptweaks.*
import nl.kallestruik.dtweaks.tweaks.dispsensertweaks.DispensersCanPlantSaplings
import nl.kallestruik.dtweaks.tweaks.miscellaneoustweaks.*
import nl.kallestruik.dtweaks.tweaks.mobtweaks.*
import org.bukkit.plugin.java.JavaPlugin
import java.io.File
import java.util.*


class DTweaks: JavaPlugin() {
    companion object {
        lateinit var plugin: DTweaks
        private lateinit var random: Random
        private lateinit var tweakManager: TweakManager
        private lateinit var util: DUtil
        private lateinit var mathHelper: MathHelper
        lateinit var reflectionUtil: ReflectionUtil
        private lateinit var trampleManager: TrampleManager
        private lateinit var commandManager: PaperCommandManager
        private lateinit var fakePlayerManager: FakePlayerManager
        private lateinit var noSpawnZoneManager: NoSpawnZoneManager
    }

    override fun onEnable() {
        /*
         * Initialize stuff
         */
        plugin = this
        random = Random()
        tweakManager = TweakManager(this)
        util = DUtil(random)
        mathHelper = MathHelper()
        reflectionUtil = ReflectionUtil()
        trampleManager = TrampleManager()
        commandManager = PaperCommandManager(this)
        fakePlayerManager = FakePlayerManager(this)
        noSpawnZoneManager = NoSpawnZoneManager()

        // Enable brigadier support on ACF
        commandManager.enableUnstableAPI("brigadier")

        registerCommandConditions()

        /*
         * Commands
         */
//        commandManager.registerCommand(CommandMobcaps(reflectionUtil))
        commandManager.registerCommand(CommandPlayer(fakePlayerManager))
        commandManager.registerCommand(CommandToggletrample(trampleManager))
        commandManager.registerCommand(CommandSpawnzones(noSpawnZoneManager))

        /*
         * Load data from disk
         */
        tweakManager.loadFromFile(File(dataFolder, Const.TWEAK_STATE_FILE_NAME))
        trampleManager.loadFromFile(File(dataFolder, Const.TRAMPLE_ENABLED_FILE_NAME))
        noSpawnZoneManager.loadFromFile(File(dataFolder, Const.NO_SPAWN_ZONE_FILE_NAME))

        /*
         * Crafting tweaks
         */
        tweakManager.registerTweak(AlternativeDispenserRecipe(this))
        tweakManager.registerTweak(CraftableDragonsBreath(this))
        tweakManager.registerTweak(CraftableNametag(this))
        tweakManager.registerTweak(CraftableSaddle(this))
        tweakManager.registerTweak(CraftableShulkerShell(this))
        tweakManager.registerTweak(CraftableSponge(this))
        tweakManager.registerTweak(IceDecompression(this))
        tweakManager.registerTweak(LogsToChests(this))
        tweakManager.registerTweak(MoreSandstone(this))
        tweakManager.registerTweak(RedyeTerracotta(this))
        tweakManager.registerTweak(WoolToString(this))
        tweakManager.registerTweak(CraftableChainmail(this))

        /*
         * Crop Tweaks
         */
        tweakManager.registerTweak(HoeHarvestArea(this))
        tweakManager.registerTweak(LilypadBonemealing(this, util))
        tweakManager.registerTweak(MobsCantTrampleCrops(this))
        tweakManager.registerTweak(PlayersCantTrampleCrops(this, trampleManager))
        tweakManager.registerTweak(SeedDropPlanting(this))
        tweakManager.registerTweak(SugarcaneBonemealing(this))

        /*
         * Dispenser Tweaks
         */
        tweakManager.registerTweak(DispensersCanPlantSaplings(this))

        /*
         * Miscellaneous Tweaks
         */
        tweakManager.registerTweak(ArmorStandArmorSwapping(this))
//        tweakManager.registerTweak(CarpetBlockPlacingProtocol(this, mathHelper))
        tweakManager.registerTweak(FakePlayers(fakePlayerManager, this))
        tweakManager.registerTweak(NoMelting(this))

        /*
         * Mob Tweaks
         */
        tweakManager.registerTweak(NoCreeperGrief(this))
        tweakManager.registerTweak(NoDoorBreaking(this))
        tweakManager.registerTweak(NoEndermanGrief(this))
        tweakManager.registerTweak(NoSpawnZones(this, noSpawnZoneManager))
    }

    override fun onDisable() {
        /*
         * Save data to disk
         */
        tweakManager.saveToFile(File(dataFolder, Const.TWEAK_STATE_FILE_NAME))
        trampleManager.saveToFile(File(dataFolder, Const.TRAMPLE_ENABLED_FILE_NAME))
        noSpawnZoneManager.saveToFile()
    }

    private fun registerCommandConditions() {
        commandManager.commandConditions.addCondition("tweakEnabled") { context ->
            if (!tweakManager.isTweakEnabled(context.getConfigValue("tweak", "")))
                throw ConditionFailedException("The tweak ${context.getConfigValue("tweak", "")} is not enabled!")
        }
    }

    fun getNamespace(): String = this.name.lowercase(Locale.ROOT)
}